Package: statgenMPP
Title: QTL Mapping for Multi Parent Populations
Version: 1.0.5
Date: 2026-05-18
Authors@R: c(person(given = "Martin", 
                    family = "Boer", 
                    email = "martin.boer@wur.nl", 
                    role = "aut",
                    comment = c(ORCID = "0000-0002-1879-4588")),
             person(given = "Wenhao", 
                    family = "Li", 
                    role = "aut",
                    comment = c(ORCID = "0000-0001-5719-5775")),        
             person(given = "Bart-Jan", 
                    family = "van Rossum", 
                    email = "bart-jan.vanrossum@wur.nl", 
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-8673-2514")),
             person(given = "Fred", 
                    family = "van Eeuwijk",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3672-2921")))        
Description: For Multi Parent Populations (MPP) Identity By Descend (IBD) 
    probabilities are computed using Hidden Markov Models. These probabilities 
    are then used in a mixed model approach for QTL Mapping as described in 
    Li et al. (<doi:10.1007/s00122-021-03919-7>).
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.6), statgenGWAS (>= 1.0.9)
Imports: data.table, foreach, ggplot2, gridExtra, LMMsolver (>= 1.0.5),
        rlang, scales, spam, statgenIBD (>= 1.0.8)
Suggests: rmarkdown, knitr, tinytest
VignetteBuilder: knitr
URL: https://biometris.github.io/statgenMPP/index.html,
        https://github.com/Biometris/statgenMPP/
BugReports: https://github.com/Biometris/statgenMPP/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 12:54:26 UTC; rossu027
Author: Martin Boer [aut] (ORCID: <https://orcid.org/0000-0002-1879-4588>),
  Wenhao Li [aut] (ORCID: <https://orcid.org/0000-0001-5719-5775>),
  Bart-Jan van Rossum [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8673-2514>),
  Fred van Eeuwijk [ctb] (ORCID: <https://orcid.org/0000-0003-3672-2921>)
Maintainer: Bart-Jan van Rossum <bart-jan.vanrossum@wur.nl>
Repository: CRAN
Date/Publication: 2026-05-18 13:20:02 UTC
Built: R 4.5.2; ; 2026-05-18 14:26:33 UTC; unix
