# Generated by roxygen2: do not edit by hand

S3method(plot,GWAS)
S3method(plot,QTLMPP)
S3method(plot,gDataMPP)
S3method(print,summary.QTLMPP)
S3method(print,summary.gDataMPP)
S3method(summary,QTLMPP)
S3method(summary,gDataMPP)
export(calcIBDMPP)
export(createGDataMPP)
export(kinshipIBD)
export(readRABBITMPP)
export(selQTLMPP)
import(grDevices)
import(graphics)
import(statgenGWAS)
importFrom(foreach,`%do%`)
importFrom(foreach,`%dopar%`)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,model.matrix)
importFrom(stats,pchisq)
importFrom(stats,setNames)
importFrom(utils,getFromNamespace)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,tail)
importFrom(utils,unzip)
