#' @importFrom utils getFromNamespace
#' @keywords internal
manhattanPlot <- getFromNamespace(x = "manhattanPlot", ns = "statgenGWAS")

#' @keywords internal
geneticMapPlot <- getFromNamespace(x = "geneticMapPlot", ns = "statgenGWAS")

#' @keywords internal
createGWAS <- getFromNamespace(x = "createGWAS", ns = "statgenGWAS")

#' @keywords internal
#' @exportS3Method
plot.GWAS <- getFromNamespace(x = "plot.GWAS", ns = "statgenGWAS")

#' @keywords internal
nearestPD <- getFromNamespace(x = "nearestPD", ns = "statgenGWAS")

#' @keywords internal
markers3DtoMat <- getFromNamespace(x = "markers3DtoMat", ns = "statgenIBD")

#' @keywords internal
allGenoPlot <- getFromNamespace(x = "allGenoPlot", ns = "statgenIBD")

#' @keywords internal
singleGenoPlot <- getFromNamespace(x = "singleGenoPlot", ns = "statgenIBD")

#' @keywords internal
pedPlot <- getFromNamespace(x = "pedPlot", ns = "statgenIBD")

#' @keywords internal
constructRinv <- getFromNamespace(x = "constructRinv", ns = "LMMsolver")

#' @keywords internal
sparseMixedModels <- getFromNamespace(x = "sparseMixedModels", ns = "LMMsolver")
